@@ -12,7 +12,7 @@
 # You should have received a copy of the GNU General Public License
 # along with Scapy. If not, see <http://www.gnu.org/licenses/>.
 
-# scapy.contrib.description = ISIS
+# scapy.contrib.description = Intermediate System to Intermediate System (ISIS)
 # scapy.contrib.status = loads
 
 """
@@ -22,6 +22,10 @@
     :copyright: 2014-2016 BENOCS GmbH, Berlin (Germany)
     :author:    Marcel Patzlaff, mpatzlaff@benocs.com
                 Michal Kaliszan, mkaliszan@benocs.com
+
+    :copyright: 2020 Metaswitch, London (UK)
+    :author:    Tom Zhu, tom.zhu@metaswitch.com
+
     :license:   GPLv2
 
         This module is free software; you can redistribute it and/or
@@ -41,13 +45,14 @@
 
         Currently it (partially) supports the packaging/encoding
         requirements of the following RFCs:
-         * RFC 1195 (only the TCP/IP related part)
-         * RFC 3358 (optional checksums)
-         * RFC 5301 (dynamic hostname extension)
-         * RFC 5302 (domain-wide prefix distribution)
-         * RFC 5303 (three-way handshake)
-         * RFC 5304 (cryptographic authentication)
-         * RFC 5308 (routing IPv6 with IS-IS)
+        * RFC 1195 (only the TCP/IP related part)
+        * RFC 3358 (optional checksums)
+        * RFC 5301 (dynamic hostname extension)
+        * RFC 5302 (domain-wide prefix distribution)
+        * RFC 5303 (three-way handshake)
+        * RFC 5304 (cryptographic authentication)
+        * RFC 5308 (routing IPv6 with IS-IS)
+        * RFC 8667 (IS-IS extensions for segment routing)
 
     :TODO:
 
@@ -65,23 +70,24 @@ import struct
 import random
 
 from scapy.config import conf
-from scapy.fields import *
-from scapy.packet import *
+from scapy.fields import BitField, BitFieldLenField, BoundStrLenField, \
+    ByteEnumField, ByteField, ConditionalField, Field, FieldLenField, \
+    FieldListField, FlagsField, IEEEFloatField, IP6PrefixField, IPField, \
+    IPPrefixField, IntField, LongField, MACField, PacketField, \
+    PacketListField, ShortField, ThreeBytesField, XIntField, XShortField
+from scapy.packet import bind_layers, Packet
 from scapy.layers.clns import network_layer_protocol_ids, register_cln_protocol
 from scapy.layers.inet6 import IP6ListField, IP6Field
 from scapy.utils import fletcher16_checkbytes
 from scapy.volatile import RandString, RandByte
-import random
 from scapy.modules.six.moves import range
-from scapy.compat import raw
+from scapy.compat import orb, hex_bytes
 
-EXT_VERSION = "v0.0.2"
-
-conf.debug_dissector = True
+EXT_VERSION = "v0.0.3"
 
 
 #######################################################################
-##  ISIS Utilities + Fields                                          ##
+#   ISIS Utilities + Fields                                           #
 #######################################################################
 def isis_area2str(area):
     return b"".join(hex_bytes(x) for x in area.split("."))
@@ -92,7 +98,7 @@ def isis_str2area(s):
         return ""
 
     numbytes = len(s[1:])
-    fmt = "%02X" + (".%02X%02X" * (numbytes // 2)) + ("" if (numbytes % 2) == 0 else ".%02X")
+    fmt = "%02X" + (".%02X%02X" * (numbytes // 2)) + ("" if (numbytes % 2) == 0 else ".%02X")  # noqa: E501
     return fmt % tuple(orb(x) for x in s)
 
 
@@ -101,7 +107,7 @@ def isis_sysid2str(sysid):
 
 
 def isis_str2sysid(s):
-    return ("%02X%02X."*3)[:-1] % tuple(orb(x) for x in s)
+    return ("%02X%02X." * 3)[:-1] % tuple(orb(x) for x in s)
 
 
 def isis_nodeid2str(nodeid):
@@ -122,6 +128,7 @@ def isis_str2lspid(s):
 
 class _ISIS_IdFieldBase(Field):
     __slots__ = ["to_str", "to_id", "length"]
+
     def __init__(self, name, default, length, to_str, to_id):
         self.to_str = to_str
         self.to_id = to_id
@@ -130,7 +137,7 @@ class _ISIS_IdFieldBase(Field):
 
     def i2m(self, pkt, x):
         if x is None:
-            return b"\0"*self.length
+            return b"\0" * self.length
 
         return self.to_str(x)
 
@@ -146,6 +153,7 @@ class _ISIS_IdFieldBase(Field):
 
 class _ISIS_RandId(RandString):
     def __init__(self, template):
+        RandString.__init__(self)
         self.bytecount = template.count("*")
         self.format = template.replace("*", "%02X")
 
@@ -162,9 +170,17 @@ class _ISIS_RandId(RandString):
 
 
 class _ISIS_RandAreaId(_ISIS_RandId):
-    def __init__(self, bytecount= None):
-        self.bytecount = random.randint(1, 13) if bytecount is None else bytecount
-        self.format = "%02X" + (".%02X%02X" * ((self.bytecount-1) // 2)) + ("" if ((self.bytecount-1) % 2) == 0 else ".%02X")
+    def __init__(self, bytecount=None):
+        template = "*" + (
+            ".**" * ((self.bytecount - 1) // 2)
+        ) + (
+            "" if ((self.bytecount - 1) % 2) == 0 else ".*"
+        )
+        super(_ISIS_RandAreaId, self).__init__(template)
+        if bytecount is None:
+            self.bytecount = random.randint(1, 13)
+        else:
+            self.bytecount = bytecount
 
 
 class ISIS_AreaIdField(Field):
@@ -183,17 +199,17 @@ class ISIS_AreaIdField(Field):
     def i2len(self, pkt, x):
         if x is None:
             return 0
-        l = len(x)
+        tmp_len = len(x)
         # l/5 is the number of dots in the Area ID
-        return (l - (l // 5)) // 2
+        return (tmp_len - (tmp_len // 5)) // 2
 
     def addfield(self, pkt, s, val):
         sval = self.i2m(pkt, val)
-        return s+struct.pack("!%is" % len(sval), sval)
+        return s + struct.pack("!%is" % len(sval), sval)
 
     def getfield(self, pkt, s):
         numbytes = self.length_from(pkt)
-        return s[numbytes:], self.m2i(pkt, struct.unpack("!%is" % numbytes, s[:numbytes])[0])
+        return s[numbytes:], self.m2i(pkt, struct.unpack("!%is" % numbytes, s[:numbytes])[0])  # noqa: E501
 
     def randval(self):
         return _ISIS_RandAreaId()
@@ -201,7 +217,7 @@ class ISIS_AreaIdField(Field):
 
 class ISIS_SystemIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 6, isis_sysid2str, isis_str2sysid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 6, isis_sysid2str, isis_str2sysid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**")
@@ -209,7 +225,7 @@ class ISIS_SystemIdField(_ISIS_IdFieldBase):
 
 class ISIS_NodeIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 7, isis_nodeid2str, isis_str2nodeid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 7, isis_nodeid2str, isis_str2nodeid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**.*")
@@ -217,7 +233,7 @@ class ISIS_NodeIdField(_ISIS_IdFieldBase):
 
 class ISIS_LspIdField(_ISIS_IdFieldBase):
     def __init__(self, name, default):
-        _ISIS_IdFieldBase.__init__(self, name, default, 8, isis_lspid2str, isis_str2lspid)
+        _ISIS_IdFieldBase.__init__(self, name, default, 8, isis_lspid2str, isis_str2lspid)  # noqa: E501
 
     def randval(self):
         return _ISIS_RandId("**.**.**.*-*")
@@ -244,7 +260,7 @@ def _ISIS_GuessTlvClass_Helper(tlv_classes, defaultname, p, **kargs):
 class _ISIS_GenericTlv_Base(Packet):
     fields_desc = [ByteField("type", 0),
                    FieldLenField("len", None, length_of="val", fmt="B"),
-                   BoundStrLenField("val", "", length_from=lambda pkt: pkt.len)]
+                   BoundStrLenField("val", "", length_from=lambda pkt: pkt.len)]  # noqa: E501
 
     def guess_payload_class(self, p):
         return conf.padding_layer
@@ -259,14 +275,14 @@ class ISIS_GenericSubTlv(_ISIS_GenericTlv_Base):
 
 
 #######################################################################
-##  ISIS Sub-TLVs for TLVs 22, 23, 141, 222, 223                     ##
+#   ISIS Sub-TLVs for TLVs 22, 23, 141, 222, 223                      #
 #######################################################################
 _isis_subtlv_classes_1 = {
-    3:  "ISIS_AdministrativeGroupSubTlv",
-    4:  "ISIS_LinkLocalRemoteIdentifiersSubTlv",
-    6:  "ISIS_IPv4InterfaceAddressSubTlv",
-    8:  "ISIS_IPv4NeighborAddressSubTlv",
-    9:  "ISIS_MaximumLinkBandwidthSubTlv",
+    3: "ISIS_AdministrativeGroupSubTlv",
+    4: "ISIS_LinkLocalRemoteIdentifiersSubTlv",
+    6: "ISIS_IPv4InterfaceAddressSubTlv",
+    8: "ISIS_IPv4NeighborAddressSubTlv",
+    9: "ISIS_MaximumLinkBandwidthSubTlv",
     10: "ISIS_MaximumReservableLinkBandwidthSubTlv",
     11: "ISIS_UnreservedBandwidthSubTlv",
     12: "ISIS_IPv6InterfaceAddressSubTlv",
@@ -275,11 +291,11 @@ _isis_subtlv_classes_1 = {
 }
 
 _isis_subtlv_names_1 = {
-    3:  "Administrative Group (Color)",
-    4:  "Link Local/Remote Identifiers",
-    6:  "IPv4 Interface Address",
-    8:  "IPv4 Neighbor Address",
-    9:  "Maximum Link Bandwidth",
+    3: "Administrative Group (Color)",
+    4: "Link Local/Remote Identifiers",
+    6: "IPv4 Interface Address",
+    8: "IPv4 Neighbor Address",
+    9: "Maximum Link Bandwidth",
     10: "Maximum Reservable Link Bandwidth",
     11: "Unreserved Bandwidth",
     12: "IPv6 Interface Address",
@@ -289,20 +305,20 @@ _isis_subtlv_names_1 = {
 
 
 def _ISIS_GuessSubTlvClass_1(p, **kargs):
-    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_1, "ISIS_GenericSubTlv", p, **kargs)
+    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_1, "ISIS_GenericSubTlv", p, **kargs)  # noqa: E501
 
 
 class ISIS_IPv4InterfaceAddressSubTlv(ISIS_GenericSubTlv):
     name = "ISIS IPv4 Interface Address (S)"
     fields_desc = [ByteEnumField("type", 6, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of= "address", fmt="B"),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
                    IPField("address", "0.0.0.0")]
 
 
 class ISIS_IPv4NeighborAddressSubTlv(ISIS_GenericSubTlv):
     name = "ISIS IPv4 Neighbor Address (S)"
     fields_desc = [ByteEnumField("type", 8, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of= "address", fmt="B"),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
                    IPField("address", "0.0.0.0")]
 
 
@@ -317,101 +333,216 @@ class ISIS_LinkLocalRemoteIdentifiersSubTlv(ISIS_GenericSubTlv):
 class ISIS_IPv6InterfaceAddressSubTlv(ISIS_GenericSubTlv):
     name = "ISIS IPv6 Interface Address (S)"
     fields_desc = [ByteEnumField("type", 12, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of= "address", fmt="B"),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
                    IP6Field("address", "::")]
 
 
 class ISIS_IPv6NeighborAddressSubTlv(ISIS_GenericSubTlv):
     name = "ISIS IPv6 Neighbor Address (S)"
     fields_desc = [ByteEnumField("type", 13, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of= "address", fmt="B"),
+                   FieldLenField("len", None, length_of="address", fmt="B"),
                    IP6Field("address", "::")]
 
+
 class ISIS_AdministrativeGroupSubTlv(ISIS_GenericSubTlv):
     name = "Administrative Group SubTLV (Color)"
     fields_desc = [ByteEnumField("code", 3, _isis_subtlv_names_1),
                    FieldLenField("len", None, length_of="admingroup", fmt="B"),
                    IPField("admingroup", "0.0.0.1")]
 
+
 class ISIS_MaximumLinkBandwidthSubTlv(ISIS_GenericSubTlv):
     name = "Maximum Link Bandwidth SubTLV"
     fields_desc = [ByteEnumField("type", 9, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of="maxbw", fmt="B"), 
-                   IEEEFloatField("maxbw", 1000)] # in B/s
+                   FieldLenField("len", None, length_of="maxbw", fmt="B"),
+                   IEEEFloatField("maxbw", 1000)]  # in B/s
+
 
 class ISIS_MaximumReservableLinkBandwidthSubTlv(ISIS_GenericSubTlv):
     name = "Maximum Reservable Link Bandwidth SubTLV"
     fields_desc = [ByteEnumField("type", 10, _isis_subtlv_names_1),
                    FieldLenField("len", None, length_of="maxrsvbw", fmt="B"),
-                   IEEEFloatField("maxrsvbw", 1000)] # in B/s
+                   IEEEFloatField("maxrsvbw", 1000)]  # in B/s
+
 
 class ISIS_UnreservedBandwidthSubTlv(ISIS_GenericSubTlv):
     name = "Unreserved Bandwidth SubTLV"
     fields_desc = [ByteEnumField("type", 11, _isis_subtlv_names_1),
                    FieldLenField("len", None, length_of="unrsvbw", fmt="B"),
-                   FieldListField("unrsvbw", [1000,1000,1000,1000,1000,1000,1000,1000], IEEEFloatField("", 1000), count_from= lambda pkt: pkt.len / 4 )] # in B/s
+                   FieldListField("unrsvbw", [1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000], IEEEFloatField("", 1000), count_from=lambda pkt: pkt.len / 4)]  # in B/s  # noqa: E501
+
 
 class ISIS_TEDefaultMetricSubTlv(ISIS_GenericSubTlv):
     name = "TE Default Metric SubTLV"
     fields_desc = [ByteEnumField("type", 18, _isis_subtlv_names_1),
-                   FieldLenField("len", None, length_of="temetric", adjust=lambda pkt,x:x-1, fmt="B"),
+                   FieldLenField("len", None, length_of="temetric", adjust=lambda pkt, x:x - 1, fmt="B"),  # noqa: E501
                    ThreeBytesField("temetric", 1000)]
 
 
 #######################################################################
-##  ISIS Sub-TLVs for TLVs 135, 235, 236, and 237                    ##
+#   ISIS Sub-TLVs for TLVs 135, 235, 236, and 237                     #
 #######################################################################
 _isis_subtlv_classes_2 = {
-    1:  "ISIS_32bitAdministrativeTagSubTlv",
-    2:  "ISIS_64bitAdministrativeTagSubTlv"
+    1: "ISIS_32bitAdministrativeTagSubTlv",
+    2: "ISIS_64bitAdministrativeTagSubTlv",
+    3: "ISIS_PrefixSegmentIdentifierSubTlv"
 }
 
 _isis_subtlv_names_2 = {
-    1:  "32-bit Administrative Tag",
-    2:  "64-bit Administrative Tag"
+    1: "32-bit Administrative Tag",
+    2: "64-bit Administrative Tag",
+    3: "Prefix Segment Identifier"
 }
 
 
 def _ISIS_GuessSubTlvClass_2(p, **kargs):
-    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_2, "ISIS_GenericSubTlv", p, **kargs)
+    return _ISIS_GuessTlvClass_Helper(_isis_subtlv_classes_2, "ISIS_GenericSubTlv", p, **kargs)  # noqa: E501
 
 
 class ISIS_32bitAdministrativeTagSubTlv(ISIS_GenericSubTlv):
     name = "ISIS 32-bit Administrative Tag (S)"
     fields_desc = [ByteEnumField("type", 1, _isis_subtlv_names_2),
-                   FieldLenField("len", None, length_of= "tags", fmt="B"),
-                   FieldListField("tags", [], IntField("", 0), count_from= lambda pkt: pkt.len // 4)]
+                   FieldLenField("len", None, length_of="tags", fmt="B"),
+                   FieldListField("tags", [], IntField("", 0), count_from=lambda pkt: pkt.len // 4)]  # noqa: E501
 
 
 class ISIS_64bitAdministrativeTagSubTlv(ISIS_GenericSubTlv):
     name = "ISIS 64-bit Administrative Tag (S)"
     fields_desc = [ByteEnumField("type", 2, _isis_subtlv_names_2),
-                   FieldLenField("len", None, length_of= "tags", fmt="B"),
-                   FieldListField("tags", [], LongField("", 0), count_from= lambda pkt: pkt.len // 8)]
+                   FieldLenField("len", None, length_of="tags", fmt="B"),
+                   FieldListField("tags", [], LongField("", 0), count_from=lambda pkt: pkt.len // 8)]  # noqa: E501
+
+
+class ISIS_PrefixSegmentIdentifierSubTlv(ISIS_GenericSubTlv):
+    name = "ISIS Prefix SID sub TLV"
+    fields_desc = [ByteEnumField("type", 3, _isis_subtlv_names_2),
+                   ByteField("len", 5),
+                   FlagsField(
+                       "flags", 0, 8,
+                       ["res1", "res2", "L", "V", "E", "P", "N", "R"]),
+                   ByteField("algorithm", 0),
+                   ConditionalField(ThreeBytesField("sid", 0),
+                                    lambda pkt: pkt.len == 5),
+                   ConditionalField(IntField("idx", 0),
+                                    lambda pkt: pkt.len == 6)]
 
 
 #######################################################################
-##  ISIS TLVs                                                        ##
+#   ISIS Sub-TLVs for TLVs 149, 150                                   #
 #######################################################################
-_isis_tlv_classes = { 
+_isis_subtlv_classes_3 = {
+    1: "ISIS_SIDLabelSubTLV"
+}
+
+_isis_subtlv_names_3 = {
+    1: "ISIS SID/Label sub TLV"
+}
+
+
+def _ISIS_GuessSubTlvClass_3(p, **kargs):
+    return _ISIS_GuessTlvClass_Helper(
+        _isis_subtlv_classes_3, "ISIS_GenericSubTlv", p, **kargs)
+
+
+class ISIS_SIDLabelSubTLV(ISIS_GenericSubTlv):
+    name = "ISIS SID Label sub TLV"
+    fields_desc = [
+        ByteEnumField("type", 1, _isis_subtlv_names_3),
+        ByteField("len", 3),
+        ConditionalField(ThreeBytesField("sid", 0),
+                         lambda pkt: pkt.len == 3),
+        ConditionalField(IntField("idx", 0),
+                         lambda pkt: pkt.len == 4)
+    ]
+
+
+#######################################################################
+#   ISIS Sub-TLVs for TLV 242                                         #
+#######################################################################
+_isis_subtlv_classes_4 = {
+    2: "ISIS_SRCapabilitiesSubTLV",
+    19: "ISIS_SRAlgorithmSubTLV",
+}
+
+_isis_subtlv_names_4 = {
+    2: "Segment Routing Capability sub TLV",
+    19: "Segment Routing Algorithm",
+}
+
+
+def _ISIS_GuessSubTlvClass_4(p, **kargs):
+    return _ISIS_GuessTlvClass_Helper(
+        _isis_subtlv_classes_4, "ISIS_GenericSubTlv", p, **kargs)
+
+
+class ISIS_SRGBDescriptorEntry(Packet):
+    name = "ISIS SRGB Descriptor"
+    fields_desc = [
+        ThreeBytesField("range", 0),
+        PacketField("sid_label", None, ISIS_SIDLabelSubTLV)
+    ]
+
+    def extract_padding(self, s):
+        return "", s
+
+
+class ISIS_SRCapabilitiesSubTLV(ISIS_GenericSubTlv):
+    name = "ISIS SR Capabilities TLV"
+    fields_desc = [
+        ByteEnumField("type", 2, _isis_subtlv_names_3),
+        FieldLenField(
+            "len",
+            None,
+            length_of="srgb_ranges",
+            adjust=lambda pkt, x: x + 1,
+            fmt="B"),
+        FlagsField(
+            "flags", 0, 8,
+            ["res1", "res2", "res3", "res4", "res5", "res6", "V", "I"]),
+        PacketListField(
+            "srgb_ranges",
+            [],
+            ISIS_SRGBDescriptorEntry,
+            length_from=lambda pkt: pkt.len - 1)
+    ]
+
+
+class ISIS_SRAlgorithmSubTLV(ISIS_GenericSubTlv):
+    name = "ISIS SR Algorithm sub TLV"
+    fields_desc = [
+        ByteEnumField("type", 19, _isis_subtlv_names_4),
+        FieldLenField("len", None, length_of="algorithms", fmt="B"),
+        FieldListField(
+            "algorithms",
+            [0],
+            ByteField("", 0),
+            count_from=lambda pkt:pkt.len)
+    ]
+
+
+#######################################################################
+#   ISIS TLVs                                                         #
+#######################################################################
+_isis_tlv_classes = {
     1: "ISIS_AreaTlv",
     2: "ISIS_IsReachabilityTlv",
     6: "ISIS_IsNeighbourTlv",
     8: "ISIS_PaddingTlv",
     9: "ISIS_LspEntryTlv",
-   10: "ISIS_AuthenticationTlv",
-   12: "ISIS_ChecksumTlv",
-   14: "ISIS_BufferSizeTlv",
-   22: "ISIS_ExtendedIsReachabilityTlv",
-  128: "ISIS_InternalIpReachabilityTlv",
-  129: "ISIS_ProtocolsSupportedTlv",
-  130: "ISIS_ExternalIpReachabilityTlv",
-  132: "ISIS_IpInterfaceAddressTlv",
-  135: "ISIS_ExtendedIpReachabilityTlv",
-  137: "ISIS_DynamicHostnameTlv",
-  232: "ISIS_Ipv6InterfaceAddressTlv",
-  236: "ISIS_Ipv6ReachabilityTlv",
-  240: "ISIS_P2PAdjacencyStateTlv"
+    10: "ISIS_AuthenticationTlv",
+    12: "ISIS_ChecksumTlv",
+    14: "ISIS_BufferSizeTlv",
+    22: "ISIS_ExtendedIsReachabilityTlv",
+    128: "ISIS_InternalIpReachabilityTlv",
+    129: "ISIS_ProtocolsSupportedTlv",
+    130: "ISIS_ExternalIpReachabilityTlv",
+    132: "ISIS_IpInterfaceAddressTlv",
+    135: "ISIS_ExtendedIpReachabilityTlv",
+    137: "ISIS_DynamicHostnameTlv",
+    232: "ISIS_Ipv6InterfaceAddressTlv",
+    236: "ISIS_Ipv6ReachabilityTlv",
+    240: "ISIS_P2PAdjacencyStateTlv",
+    242: "ISIS_RouterCapabilityTlv"
 }
 
 _isis_tlv_names = {
@@ -421,54 +552,54 @@ _isis_tlv_names = {
     7: "Instance Identifier TLV",
     8: "Padding TLV",
     9: "LSP Entries TLV",
-   10: "Authentication TLV",
-   12: "Optional Checksum TLV",
-   13: "Purge Originator Identification TLV", 
-   14: "LSP Buffer Size TLV",
-   22: "Extended IS-Reachability TLV",
-   23: "IS Neighbour Attribute TLV",
-   24: "IS Alias ID",
-  128: "IP Internal Reachability TLV",
-  129: "Protocols Supported TLV",
-  130: "IP External Reachability TLV",
-  131: "Inter-Domain Routing Protocol Information TLV",
-  132: "IP Interface Address TLV",
-  134: "Traffic Engineering Router ID TLV",
-  135: "Extended IP Reachability TLV",
-  137: "Dynamic Hostname TLV",
-  138: "GMPLS Shared Risk Link Group TLV",
-  139: "IPv6 Shared Risk Link Group TLV",
-  140: "IPv6 Traffic Engineering Router ID TLV",
-  141: "Inter-AS Reachability Information TLV",
-  142: "Group Address TLV",
-  143: "Multi-Topology-Aware Port Capability TLV",
-  144: "Multi-Topology Capability TLV",
-  145: "TRILL Neighbour TLV",
-  147: "MAC-Reachability TLV",
-  148: "BFD-Enabled TLV",
-  211: "Restart TLV",
-  222: "Multi-Topology Intermediate Systems TLV",
-  223: "Multi-Topology IS Neighbour Attributes TLV",
-  229: "Multi-Topology TLV",
-  232: "IPv6 Interface Address TLV",
-  233: "IPv6 Global Interface Address TLV",
-  235: "Multi-Topology IPv4 Reachability TLV",
-  236: "IPv6 Reachability TLV",
-  237: "Multi-Topology IPv6 Reachability TLV",
-  240: "Point-to-Point Three-Way Adjacency TLV",
-  242: "IS-IS Router Capability TLV",
-  251: "Generic Information TLV"
+    10: "Authentication TLV",
+    12: "Optional Checksum TLV",
+    13: "Purge Originator Identification TLV",
+    14: "LSP Buffer Size TLV",
+    22: "Extended IS-Reachability TLV",
+    23: "IS Neighbour Attribute TLV",
+    24: "IS Alias ID",
+    128: "IP Internal Reachability TLV",
+    129: "Protocols Supported TLV",
+    130: "IP External Reachability TLV",
+    131: "Inter-Domain Routing Protocol Information TLV",
+    132: "IP Interface Address TLV",
+    134: "Traffic Engineering Router ID TLV",
+    135: "Extended IP Reachability TLV",
+    137: "Dynamic Hostname TLV",
+    138: "GMPLS Shared Risk Link Group TLV",
+    139: "IPv6 Shared Risk Link Group TLV",
+    140: "IPv6 Traffic Engineering Router ID TLV",
+    141: "Inter-AS Reachability Information TLV",
+    142: "Group Address TLV",
+    143: "Multi-Topology-Aware Port Capability TLV",
+    144: "Multi-Topology Capability TLV",
+    145: "TRILL Neighbour TLV",
+    147: "MAC-Reachability TLV",
+    148: "BFD-Enabled TLV",
+    211: "Restart TLV",
+    222: "Multi-Topology Intermediate Systems TLV",
+    223: "Multi-Topology IS Neighbour Attributes TLV",
+    229: "Multi-Topology TLV",
+    232: "IPv6 Interface Address TLV",
+    233: "IPv6 Global Interface Address TLV",
+    235: "Multi-Topology IPv4 Reachability TLV",
+    236: "IPv6 Reachability TLV",
+    237: "Multi-Topology IPv6 Reachability TLV",
+    240: "Point-to-Point Three-Way Adjacency TLV",
+    242: "IS-IS Router Capability TLV",
+    251: "Generic Information TLV"
 }
 
 
 def _ISIS_GuessTlvClass(p, **kargs):
-    return _ISIS_GuessTlvClass_Helper(_isis_tlv_classes, "ISIS_GenericTlv", p, **kargs)
+    return _ISIS_GuessTlvClass_Helper(_isis_tlv_classes, "ISIS_GenericTlv", p, **kargs)  # noqa: E501
 
 
 class ISIS_AreaEntry(Packet):
     name = "ISIS Area Entry"
     fields_desc = [FieldLenField("arealen", None, length_of="areaid", fmt="B"),
-                   ISIS_AreaIdField("areaid", "49", length_from=lambda pkt: pkt.arealen)]
+                   ISIS_AreaIdField("areaid", "49", length_from=lambda pkt: pkt.arealen)]  # noqa: E501
 
     def extract_padding(self, s):
         return "", s
@@ -477,16 +608,16 @@ class ISIS_AreaEntry(Packet):
 class ISIS_AreaTlv(ISIS_GenericTlv):
     name = "ISIS Area TLV"
     fields_desc = [ByteEnumField("type", 1, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "areas", fmt="B"),
-                   PacketListField("areas", [], ISIS_AreaEntry, length_from=lambda x: x.len)]
+                   FieldLenField("len", None, length_of="areas", fmt="B"),
+                   PacketListField("areas", [], ISIS_AreaEntry, length_from=lambda x: x.len)]  # noqa: E501
 
 
 class ISIS_AuthenticationTlv(ISIS_GenericTlv):
     name = "ISIS Authentication TLV"
     fields_desc = [ByteEnumField("type", 10, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "password", adjust=lambda pkt,x: x + 1, fmt="B"),
+                   FieldLenField("len", None, length_of="password", adjust=lambda pkt, x: x + 1, fmt="B"),  # noqa: E501
                    ByteEnumField("authtype", 1, {1: "Plain", 17: "HMAC-MD5"}),
-                   BoundStrLenField("password", "", maxlen= 254, length_from=lambda pkt: pkt.len - 1)]
+                   BoundStrLenField("password", "", maxlen=254, length_from=lambda pkt: pkt.len - 1)]  # noqa: E501
 
 
 class ISIS_BufferSizeTlv(ISIS_GenericTlv):
@@ -506,8 +637,8 @@ class ISIS_ChecksumTlv(ISIS_GenericTlv):
 class ISIS_DynamicHostnameTlv(ISIS_GenericTlv):
     name = "ISIS Dynamic Hostname TLV"
     fields_desc = [ByteEnumField("type", 137, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "hostname", fmt="B"),
-                   BoundStrLenField("hostname", "", length_from=lambda pkt: pkt.len)]
+                   FieldLenField("len", None, length_of="hostname", fmt="B"),
+                   BoundStrLenField("hostname", "", length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_ExtendedIpPrefix(Packet):
@@ -517,25 +648,27 @@ class ISIS_ExtendedIpPrefix(Packet):
         BitField("updown", 0, 1),
         BitField("subtlvindicator", 0, 1),
         BitFieldLenField("pfxlen", None, 6, length_of="pfx"),
-        IPPrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),
-        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt= "B"), lambda pkt: pkt.subtlvindicator == 1),
-        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)
+        IPPrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),  # noqa: E501
+        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"), lambda pkt: pkt.subtlvindicator == 1),  # noqa: E501
+        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)  # noqa: E501
     ]
 
     def extract_padding(self, s):
         return "", s
 
+
 class ISIS_TERouterIDTlv(ISIS_GenericTlv):
     name = "ISIS TE Router ID TLV"
     fields_desc = [ByteEnumField("type", 134, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "routerid", fmt="B"),
+                   FieldLenField("len", None, length_of="routerid", fmt="B"),
                    IPField("routerid", "0.0.0.0")]
- 
+
+
 class ISIS_ExtendedIpReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS Extended IP Reachability TLV"
     fields_desc = [ByteEnumField("type", 135, _isis_tlv_names),
                    FieldLenField("len", None, length_of="pfxs", fmt="B"),
-                   PacketListField("pfxs", [], ISIS_ExtendedIpPrefix, length_from= lambda pkt: pkt.len)]
+                   PacketListField("pfxs", [], ISIS_ExtendedIpPrefix, length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_ExtendedIsNeighbourEntry(Packet):
@@ -543,8 +676,8 @@ class ISIS_ExtendedIsNeighbourEntry(Packet):
     fields_desc = [
         ISIS_NodeIdField("neighbourid", "0102.0304.0506.07"),
         ThreeBytesField("metric", 1),
-        FieldLenField("subtlvslen", None, length_of="subtlvs", fmt= "B"),
-        PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_1, length_from=lambda x: x.subtlvslen)
+        FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"),
+        PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_1, length_from=lambda x: x.subtlvslen)  # noqa: E501
     ]
 
     def extract_padding(self, s):
@@ -555,14 +688,14 @@ class ISIS_ExtendedIsReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS Extended IS Reachability TLV"
     fields_desc = [ByteEnumField("type", 22, _isis_tlv_names),
                    FieldLenField("len", None, length_of="neighbours", fmt="B"),
-                   PacketListField("neighbours", [], ISIS_ExtendedIsNeighbourEntry, length_from=lambda x: x.len)]
+                   PacketListField("neighbours", [], ISIS_ExtendedIsNeighbourEntry, length_from=lambda x: x.len)]  # noqa: E501
 
 
 class ISIS_IpInterfaceAddressTlv(ISIS_GenericTlv):
     name = "ISIS IP Interface Address TLV"
     fields_desc = [ByteEnumField("type", 132, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "addresses", fmt="B"),
-                   FieldListField("addresses", [], IPField("", "0.0.0.0"), count_from= lambda pkt: pkt.len // 4)]
+                   FieldLenField("len", None, length_of="addresses", fmt="B"),
+                   FieldListField("addresses", [], IPField("", "0.0.0.0"), count_from=lambda pkt: pkt.len // 4)]  # noqa: E501
 
 
 class ISIS_Ipv6InterfaceAddressTlv(ISIS_GenericTlv):
@@ -583,9 +716,9 @@ class ISIS_Ipv6Prefix(Packet):
         BitField("subtlvindicator", 0, 1),
         BitField("reserved", 0, 5),
         FieldLenField("pfxlen", None, length_of="pfx", fmt="B"),
-        IP6PrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),
-        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt= "B"), lambda pkt: pkt.subtlvindicator == 1),
-        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)
+        IP6PrefixField("pfx", None, wordbytes=1, length_from=lambda x: x.pfxlen),  # noqa: E501
+        ConditionalField(FieldLenField("subtlvslen", None, length_of="subtlvs", fmt="B"), lambda pkt: pkt.subtlvindicator == 1),  # noqa: E501
+        ConditionalField(PacketListField("subtlvs", [], _ISIS_GuessSubTlvClass_2, length_from=lambda x: x.subtlvslen), lambda pkt: pkt.subtlvindicator == 1)  # noqa: E501
     ]
 
     def extract_padding(self, s):
@@ -593,17 +726,17 @@ class ISIS_Ipv6Prefix(Packet):
 
 
 class ISIS_Ipv6ReachabilityTlv(ISIS_GenericTlv):
-    name= "ISIS IPv6 Reachability TLV"
+    name = "ISIS IPv6 Reachability TLV"
     fields_desc = [ByteEnumField("type", 236, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "pfxs", fmt="B"),
-                   PacketListField("pfxs", [], ISIS_Ipv6Prefix, length_from= lambda pkt: pkt.len)]
+                   FieldLenField("len", None, length_of="pfxs", fmt="B"),
+                   PacketListField("pfxs", [], ISIS_Ipv6Prefix, length_from=lambda pkt: pkt.len)]  # noqa: E501
 
 
 class ISIS_IsNeighbourTlv(ISIS_GenericTlv):
     name = "ISIS IS Neighbour TLV"
     fields_desc = [ByteEnumField("type", 6, _isis_tlv_names),
-                   FieldLenField("len", None, length_of= "neighbours", fmt="B"),
-                   FieldListField("neighbours", [], MACField("", "00.00.00.00.00.00"), count_from= lambda pkt: pkt.len // 6)]
+                   FieldLenField("len", None, length_of="neighbours", fmt="B"),
+                   FieldListField("neighbours", [], MACField("", "00.00.00.00.00.00"), count_from=lambda pkt: pkt.len // 6)]  # noqa: E501
 
 
 class ISIS_LspEntry(Packet):
@@ -622,7 +755,7 @@ class ISIS_LspEntryTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 9, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_LspEntry, count_from=lambda pkt: pkt.len // 16)
+        PacketListField("entries", [], ISIS_LspEntry, count_from=lambda pkt: pkt.len // 16)  # noqa: E501
     ]
 
 
@@ -643,11 +776,11 @@ class _AdjacencyStateTlvLenField(Field):
 class ISIS_P2PAdjacencyStateTlv(ISIS_GenericTlv):
     name = "ISIS P2P Adjacency State TLV"
     fields_desc = [ByteEnumField("type", 240, _isis_tlv_names),
-               _AdjacencyStateTlvLenField("len", None, fmt="B"),
-               ByteEnumField("state", "Down", {0x2 : "Down", 0x1 : "Initialising", 0x0 : "Up"}),
-               ConditionalField(IntField("extlocalcircuitid", None), lambda pkt: pkt.len >= 5),
-               ConditionalField(ISIS_SystemIdField("neighboursystemid", None), lambda pkt: pkt.len >= 11),
-               ConditionalField(IntField("neighbourextlocalcircuitid", None), lambda pkt: pkt.len == 15)]
+                   _AdjacencyStateTlvLenField("len", None, fmt="B"),
+                   ByteEnumField("state", "Down", {0x2: "Down", 0x1: "Initialising", 0x0: "Up"}),  # noqa: E501
+                   ConditionalField(IntField("extlocalcircuitid", None), lambda pkt: pkt.len >= 5),  # noqa: E501
+                   ConditionalField(ISIS_SystemIdField("neighboursystemid", None), lambda pkt: pkt.len >= 11),  # noqa: E501
+                   ConditionalField(IntField("neighbourextlocalcircuitid", None), lambda pkt: pkt.len == 15)]  # noqa: E501
 
 
 # TODO dynamically allocate sufficient size
@@ -665,12 +798,34 @@ class ISIS_ProtocolsSupportedTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 129, _isis_tlv_names),
         FieldLenField("len", None, count_of="nlpids", fmt="B"),
-        FieldListField("nlpids", [], ByteEnumField("", "IPv4", network_layer_protocol_ids), count_from=lambda pkt: pkt.len)
+        FieldListField("nlpids", [], ByteEnumField("", "IPv4", network_layer_protocol_ids), count_from=lambda pkt: pkt.len)  # noqa: E501
+    ]
+
+
+class ISIS_RouterCapabilityTlv(ISIS_GenericTlv):
+    name = "ISIS Router Capability TLV"
+    fields_desc = [
+        ByteEnumField("type", 242, _isis_tlv_names),
+        FieldLenField(
+            "len",
+            None,
+            length_of="subtlvs",
+            adjust=lambda pkt, x: x + 5,
+            fmt="B"),
+        IPField("routerid", "0.0.0.0"),
+        FlagsField(
+            "flags", 0, 8,
+            ["S", "D", "res1", "res2", "res3", "res4", "res5", "res6"]),
+        PacketListField(
+            "subtlvs",
+            [],
+            _ISIS_GuessSubTlvClass_4,
+            length_from=lambda pkt: pkt.len - 5)
     ]
 
 
 #######################################################################
-##  ISIS Old-Style TLVs                                              ##
+#   ISIS Old-Style TLVs                                               #
 #######################################################################
 
 class ISIS_IpReachabilityEntry(Packet):
@@ -691,16 +846,16 @@ class ISIS_InternalIpReachabilityTlv(ISIS_GenericTlv):
     fields_desc = [
         ByteEnumField("type", 128, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)
+        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)  # noqa: E501
     ]
 
 
-class ISIS_ExternalIpReachabilityTLV(ISIS_GenericTlv):
+class ISIS_ExternalIpReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS External IP Reachability TLV"
     fields_desc = [
         ByteEnumField("type", 130, _isis_tlv_names),
         FieldLenField("len", None, length_of="entries", fmt="B"),
-        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)
+        PacketListField("entries", [], ISIS_IpReachabilityEntry, count_from=lambda x: x.len // 12)  # noqa: E501
     ]
 
 
@@ -720,13 +875,14 @@ class ISIS_IsReachabilityTlv(ISIS_GenericTlv):
     name = "ISIS IS Reachability TLV"
     fields_desc = [
         ByteEnumField("type", 2, _isis_tlv_names),
-        FieldLenField("len", None, fmt="B", length_of="neighbours", adjust=lambda pkt,x: x+1),
+        FieldLenField("len", None, fmt="B", length_of="neighbours", adjust=lambda pkt, x: x + 1),  # noqa: E501
         ByteField("virtual", 0),
-        PacketListField("neighbours", [], ISIS_IsReachabilityEntry, count_from=lambda x: (x.len - 1) // 11)
+        PacketListField("neighbours", [], ISIS_IsReachabilityEntry, count_from=lambda x: (x.len - 1) // 11)  # noqa: E501
     ]
 
+
 #######################################################################
-##  ISIS PDU Packets                                                 ##
+#   ISIS PDU Packets                                                  #
 #######################################################################
 _isis_pdu_names = {
     15: "L1 LAN Hello",
@@ -762,7 +918,7 @@ class ISIS_CommonHdr(Packet):
         if checksumInfo is not None:
             (cbegin, cpos) = checksumInfo
             checkbytes = fletcher16_checkbytes(pdu[cbegin:], (cpos - cbegin))
-            pdu = pdu[:cpos] + checkbytes + pdu[cpos+2:]
+            pdu = pdu[:cpos] + checkbytes + pdu[cpos + 2:]
 
         return pdu
 
@@ -785,12 +941,12 @@ class _ISIS_PduBase(Packet):
 
 class _ISIS_PduLengthField(FieldLenField):
     def __init__(self):
-        FieldLenField.__init__(self, "pdulength", None, length_of="tlvs", adjust=lambda pkt,x: x + pkt.underlayer.hdrlen)
+        FieldLenField.__init__(self, "pdulength", None, length_of="tlvs", adjust=lambda pkt, x: x + pkt.underlayer.hdrlen)  # noqa: E501
 
 
 class _ISIS_TlvListField(PacketListField):
     def __init__(self):
-        PacketListField.__init__(self, "tlvs", [], _ISIS_GuessTlvClass, length_from= lambda pkt: pkt.pdulength - pkt.underlayer.hdrlen)
+        PacketListField.__init__(self, "tlvs", [], _ISIS_GuessTlvClass, length_from=lambda pkt: pkt.pdulength - pkt.underlayer.hdrlen)  # noqa: E501
 
 
 class _ISIS_LAN_HelloBase(_ISIS_PduBase):
@@ -833,7 +989,7 @@ class _ISIS_LSP_Base(_ISIS_PduBase):
         ISIS_LspIdField("lspid", "0102.0304.0506.00-00"),
         XIntField("seqnum", 0x00000001),
         XShortField("checksum", None),
-        FlagsField("typeblock", 0x03, 8, ["L1", "L2", "OL", "ADef", "ADel", "AExp", "AErr", "P"]),
+        FlagsField("typeblock", 0x03, 8, ["L1", "L2", "OL", "ADef", "ADel", "AExp", "AErr", "P"]),  # noqa: E501
         _ISIS_TlvListField()
     ]
 
@@ -913,6 +1069,7 @@ class ISIS_L2_PSNP(_ISIS_PSNP_Base):
     def answers(self, other):
         return _snp_answers(self, other, "ISIS_L2_LSP")
 
+
 register_cln_protocol(0x83, ISIS_CommonHdr)
 bind_layers(ISIS_CommonHdr, ISIS_L1_LAN_Hello, hdrlen=27, pdutype=15)
 bind_layers(ISIS_CommonHdr, ISIS_L2_LAN_Hello, hdrlen=27, pdutype=16)
@@ -923,4 +1080,3 @@ bind_layers(ISIS_CommonHdr, ISIS_L1_CSNP, hdrlen=33, pdutype=24)
 bind_layers(ISIS_CommonHdr, ISIS_L2_CSNP, hdrlen=33, pdutype=25)
 bind_layers(ISIS_CommonHdr, ISIS_L1_PSNP, hdrlen=17, pdutype=26)
 bind_layers(ISIS_CommonHdr, ISIS_L2_PSNP, hdrlen=17, pdutype=27)
-
